AY3891x Library
====================

Arduino library for General Instrument AY-3-8910 / AY-3-8912 Programmable Sound Generator chip.

The AY-3-8913 variant has the same physical silicon as the other chips, but has a slightly different programming interface which uses a Chip Select signal. *The 8913 variant is not currently supported by the library. A future iteration of this library will likely include support.*

The AY-3-8910 and AY-3-8912 have the same programming interface. The 8910 has two 8-bit I/O ports and the 8912 has a single 8-bit I/O port (and correspondingly fewer pins). These general purpose I/O pins are not related to the sound generation functions. This library has only been tested with the 8910 variant of the chip, but should also work with the 8912.

```
Device     Package Pins  General Purpose I/O Pins
---------  ------------  ------------------------
AY-3-8910       40            16
AY-3-8912       28             8
AY-3-8913       24             0  // Not currently supported
```

There are several [variants][3] of the chip from other manufacturers, examples of which are the Yamaha YM2149F and YM3439 and Toshiba T7766A. These variants are probably compatible with this library but have not been tested.

Usage
-----

*Refer to the sketches in the `examples` folder.*

1. Include the library header file:  
```
#include "AY3891x.h"
```
2. Instantiate the object. There are two forms of the constructor. The first one supports all of the chip's interface pins. The second one allows for a slightly simplified format when using only the minimal pins necessary to interface with the chip.
```
AY3891x(byte  DA7,  byte DA6, byte DA5, byte DA4, byte DA3, byte DA2, byte DA1, byte DA0,
        byte  BDIR, byte BC2, byte BC1,
        byte  A9,   byte A8,
        byte  reset, byte clock);
```
```
AY3891x(byte  DA7,  byte DA6, byte DA5, byte DA4, byte DA3, byte DA2, byte DA1, byte DA0,
        byte  BDIR, byte BC2, byte BC1);
```
```
// Example constructor using the simpler form
AY3891x psg(4, 5, 6, 7, 8, 10, 11, 12, 2, A5, 3);
```
Use the enumeration `AY3891::NO_PIN` if any of the pins required by the constructor are not connected.

3. Initialize the object.
```
psg.begin();
```

4. Use the read() and write() methods to program the chip:
```
psg.write(AY3891x::Enable_Reg, 0x3F);  
psg.read(AY3891x::IO_Port_A_Reg;
```

5. See the library source code and example sketches for other available methods and usage.

Example Sketches
----------------
**EX1 - Find Address**  
The AY-3-8910 chip typically has a binary address of `01 0000 xxxx`,
where `xxxx` represents the specific register you are addressing.
It is possible to request a different mask-programmed address from the factory.
This example code determines the chip's address by writing a value
to one of the chip's registers and reads it back to see if it matches.
If it matches, then we found the address. If not, then the next address
is checked.

**EX2 - Test Registers**  
Writes a value to each of the chip's registers and reads it back to see if it matches.

**EX3 - Simple Tone**  
This sketch will play through the middle-octave notes C4 to C5.
Note that an external clock is required for the chip's audio functions to work. This can be [generated by the microcontroller][2] or other hardware clock circuit.

**EX4 - I/O Ports**  
This sketch demonstrates the use of the I/O ports on the AY-3-8910 or -8912.

**EX5 - Serial Commands**  
This sketch allows you to send commands to the AY-3-891x chip through the serial port. It makes it easy to experiment with the various sound generation capabilities of the chip.

References
----------
+ AY-3-891x [datasheet][1]
+ Info from the Synth DIY [wiki][4]

[1]: http://map.grauw.nl/resources/sound/generalinstrument_ay-3-8910.pdf
[2]: https://www.instructables.com/Arduino-MIDI-Chiptune-Synthesizer/
[3]: https://en.wikipedia.org/wiki/General_Instrument_AY-3-8910#Variants
[4]: https://sdiy.info/wiki/General_Instrument_AY-3-8910
